#!/usr/bin/python
# -*- coding: utf-8 -*-

import bitstring


def bits_to_group(array, size):
    """
    Convert a bitstring.BitArray or array of individual bits to an array
    of groups of bits of a given size.

    Raises exception if the length of the array is not a multiple of the
    given group size.
    """
    # sanity checking
    if divmod(len(array), size)[1] is not 0:
        raise ValueError('The given BitArray or array instance size is not '
                         'a multiple of the given group size.')
    e = enumerate(array)
    result = []
    for i in e:
        bits = bitstring.BitArray([i[1]])
        for x in range(1, size):
            bits += bitstring.BitArray([e.next()[1]])
        result.append(bits)
    return result



#!/usr/bin/python
# -*- coding: utf-8 -*-

import itertools
import math


def permute(length=8, n=0, wrap_around=False):
    """
    Return a tuple of integers representing permutation n, size of length.

    If wrap_around is set to True, if n is bigger than !length, then this
    function will return permutation n modulo !length
    (simulates going full circle).
    """
    # sanity-checking
    if not n >= 0:
        raise ValueError('Invalid permutation argument: ' + str(n))
    if not isinstance(length, int):
        raise ValueError('Expected integer for permutation length')
    set_length = math.factorial(length)
    if n > (set_length - 1):
        if wrap_around:
            n %= set_length
        else:
            raise ValueError('Requested permutation number beyond the range '
                             'of this permutation set, and wrap_around is set '
                             'to False.')
    # efficiency: if n is greater than half of set_length then reverse the
    # iteration list and iterate backwards:
    # FIXME: For some reason it appears that for large set_lengths will always
    # use forward iteration rather than backwards iteration, giving the
    # impression that the test below is failing.
    if n > (set_length / 2):
        # retrieve a permutation iterator of reversed array:
        p = itertools.permutations(reversed([i for i in range(0, length)]))
        # count backwards
        count = (set_length - 1)
        inc = -1
    else:
        # retrieve a permutation iterator:
        p = itertools.permutations([i for i in range(0, length)])
        # count forwards
        count = 0
        inc = 1

    # iterate backwards or forwards until the correct permutation is found:
    for result in p:
        if count == n:
            return result
        else:
            count += inc



#!/usr/bin/python

from setuptools import setup

setup(
    name='pyscrambler',
    version='0.0.6',
    description='Concept data scrambler using permutations as a basis',
    url='https://github.com/saxbophone/pyscrambler',
    author='Joshua Saxby',
    author_email='joshua.a.saxby@gmail.com',
    license='MIT',
    packages=['pyscrambler'],
    install_requires=['bitstring==3.1.3'],
    extras_require={
        'test': ['coverage==3.7.1'],
    },
    zip_safe=False
)



#!/usr/bin/python
# -*- coding: utf-8 -*-


def re_arrange(data_array, order_array):
    """
    Re-arrange the items in data_array to be
    in the order specified by order_array.
    """
    # sanity checking
    if not hasattr(data_array, '__iter__'):
        raise TypeError('Non-iterable type passed as data_array argument.')
    if not hasattr(order_array, '__iter__'):
        raise TypeError('Non-iterable type passed as order_array argument.')
    if len(order_array) is not len(data_array):
        raise ValueError('data_array and order_array length do not match.')
    length = len(data_array)
    new_array = [None] * length
    for norm, index in enumerate(order_array):
        new_array[norm] = data_array[index]
    return new_array


def reverse_order(order_array):
    """
    Returns the effective 'inverse' of the given order array, that is by
    returning the order array that would be needed to re-arrange back to the
    correct order.
    """
    # sanity checking
    if not hasattr(order_array, '__iter__'):
        raise TypeError('Non-iterable type passed as order_array argument.')
    length = len(order_array)
    new_array = [None] * length
    for index, replace in enumerate(order_array):
        new_array[replace] = index
    return new_array
